<?php

namespace Drupal\contextual_reports;

use Drupal\contextual_reports\Entity\ContextualReportEntity;

class ContextualReportsUtilities {

  /*
   ****************************************************************************
   * Report Entities
   * Report entities are snapshots of report data, so that reports may be
   * archived and re-loaded at a future date. This allows for easy retrieval
   * and comparison of archived report data.
   ****************************************************************************
   */

  /**
   * @param $context
   * @param $report
   * @param array $params
   * @param array $entity_data
   * @return int
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function createReportEntity($context, $report, array $params, array $entity_data = []) {
    $data = ContextualReportsUtilities::generateReportData($context, $report, $params);
    $entity_data['context'] = $context;
    $entity_data['report'] = $report;
    $entity_data['params'] = $params;
    $report = ContextualReportEntity::create($entity_data)->save();
    return $report;
  }

  /*
   ****************************************************************************
   * Reports
   ****************************************************************************
   */

  public static function getEntities($context, array $params) {
    $plugin_manager = \Drupal::service('plugin.manager.contextual_report_context');
    $plugin_definitions = $plugin_manager->getDefinitions();
    if (isset($plugin_definitions[$context]['class'])) {
      return $plugin_definitions[$context]['class']::getEntities($params);
    }
    // We have an invalid context plugin.
    return [];
  }

  public static function generateReportData($context, $report, array $params) {
    $entities = ContextualReportsUtilities::getEntities($context, $params);
    if (!empty($entities)) {
      $plugin_manager = \Drupal::service('plugin.manager.contextual_report');
      $plugin_definitions = $plugin_manager->getDefinitions();
      if (isset($plugin_definitions[$report]['class'])) {
        return $plugin_definitions[$report]['class']::generateReportData($entities, $params);
      }
    }
    // We have no data, or we have an invalid report plugin.
    return [];
  }

  public static function generateReport($context, $report, array $params) {
    $report_data = ContextualReportsUtilities::generateReportData($context, $report, $params);
    if (!empty($report_data)) {
      $plugin_manager = \Drupal::service('plugin.manager.contextual_report');
      $plugin_definitions = $plugin_manager->getDefinitions();
      if (isset($plugin_definitions[$report]['class'])) {
        return $plugin_definitions[$report]['class']::generateReport($report_data, $params);
      }
    }
    // We have no data, or we have an invalid report plugin.
    return [];
  }



}